#define GP_LOGGING_LEVEL 3
#include "W3CX11.h"


namespace gp {
    GPAPI std::unordered_map<std::string, unique_ptr<ColorHex>> colors;

    void initW3CX11() {
        GP_CORE_DEBUG("gp::initW3CX11()");

        colors["aliceblue"] = make_unique<ColorHex>("f0f8ff");
        colors["antiquewhite"] = make_unique<ColorHex>("faebd7");
        colors["aqua"] = make_unique<ColorHex>("00ffff");
        colors["aquamarine"] = make_unique<ColorHex>("7fffd4");
        colors["azure"] = make_unique<ColorHex>("f0ffff");
        colors["beige"] = make_unique<ColorHex>("f5f5dc");
        colors["bisque"] = make_unique<ColorHex>("ffe4c4");
        colors["black"] = make_unique<ColorHex>("000000");
        colors["blanchedalmond"] = make_unique<ColorHex>("ffebcd");
        colors["blue"] = make_unique<ColorHex>("0000ff");
        colors["blueviolet"] = make_unique<ColorHex>("8a2be2");
        colors["brown"] = make_unique<ColorHex>("a52a2a");
        colors["burlywood"] = make_unique<ColorHex>("deb887");
        colors["cadetblue"] = make_unique<ColorHex>("5f9ea0");
        colors["chartreuse"] = make_unique<ColorHex>("7fff00");
        colors["chocolate"] = make_unique<ColorHex>("d2691e");
        colors["coral"] = make_unique<ColorHex>("ff7f50");
        colors["cornflower"] = make_unique<ColorHex>("6495ed");
        colors["cornflowerblue"] = make_unique<ColorHex>("6495ed");
        colors["cornsilk"] = make_unique<ColorHex>("fff8dc");
        colors["crimson"] = make_unique<ColorHex>("dc143c");
        colors["cyan"] = make_unique<ColorHex>("00ffff");
        colors["darkblue"] = make_unique<ColorHex>("00008b");
        colors["darkcyan"] = make_unique<ColorHex>("008b8b");
        colors["darkgoldenrod"] = make_unique<ColorHex>("b8860b");
        colors["darkgray"] = make_unique<ColorHex>("a9a9a9");
        colors["darkgreen"] = make_unique<ColorHex>("006400");
        colors["darkgrey"] = make_unique<ColorHex>("a9a9a9");
        colors["darkkhaki"] = make_unique<ColorHex>("bdb76b");
        colors["darkmagenta"] = make_unique<ColorHex>("8b008b");
        colors["darkolivegreen"] = make_unique<ColorHex>("556b2f");
        colors["darkorange"] = make_unique<ColorHex>("ff8c00");
        colors["darkorchid"] = make_unique<ColorHex>("9932cc");
        colors["darkred"] = make_unique<ColorHex>("8b0000");
        colors["darksalmon"] = make_unique<ColorHex>("e9967a");
        colors["darkseagreen"] = make_unique<ColorHex>("8fbc8f");
        colors["darkslateblue"] = make_unique<ColorHex>("483d8b");
        colors["darkslategray"] = make_unique<ColorHex>("2f4f4f");
        colors["darkslategrey"] = make_unique<ColorHex>("2f4f4f");
        colors["darkturquoise"] = make_unique<ColorHex>("00ced1");
        colors["darkviolet"] = make_unique<ColorHex>("9400d3");
        colors["deeppink"] = make_unique<ColorHex>("ff1493");
        colors["deepskyblue"] = make_unique<ColorHex>("00bfff");
        colors["dimgray"] = make_unique<ColorHex>("696969");
        colors["dimgrey"] = make_unique<ColorHex>("696969");
        colors["dodgerblue"] = make_unique<ColorHex>("1e90ff");
        colors["firebrick"] = make_unique<ColorHex>("b22222");
        colors["floralwhite"] = make_unique<ColorHex>("fffaf0");
        colors["forestgreen"] = make_unique<ColorHex>("228b22");
        colors["fuchsia"] = make_unique<ColorHex>("ff00ff");
        colors["gainsboro"] = make_unique<ColorHex>("dcdcdc");
        colors["ghostwhite"] = make_unique<ColorHex>("f8f8ff");
        colors["gold"] = make_unique<ColorHex>("ffd700");
        colors["goldenrod"] = make_unique<ColorHex>("daa520");
        colors["gray"] = make_unique<ColorHex>("808080");
        colors["green"] = make_unique<ColorHex>("008000");
        colors["greenyellow"] = make_unique<ColorHex>("adff2f");
        colors["grey"] = make_unique<ColorHex>("808080");
        colors["honeydew"] = make_unique<ColorHex>("f0fff0");
        colors["hotpink"] = make_unique<ColorHex>("ff69b4");
        colors["indianred"] = make_unique<ColorHex>("cd5c5c");
        colors["indigo"] = make_unique<ColorHex>("4b0082");
        colors["ivory"] = make_unique<ColorHex>("fffff0");
        colors["khaki"] = make_unique<ColorHex>("f0e68c");
        colors["laserlemon"] = make_unique<ColorHex>("ffff54");
        colors["lavender"] = make_unique<ColorHex>("e6e6fa");
        colors["lavenderblush"] = make_unique<ColorHex>("fff0f5");
        colors["lawngreen"] = make_unique<ColorHex>("7cfc00");
        colors["lemonchiffon"] = make_unique<ColorHex>("fffacd");
        colors["lightblue"] = make_unique<ColorHex>("add8e6");
        colors["lightcoral"] = make_unique<ColorHex>("f08080");
        colors["lightcyan"] = make_unique<ColorHex>("e0ffff");
        colors["lightgoldenrod"] = make_unique<ColorHex>("fafad2");
        colors["lightgoldenrodyellow"] = make_unique<ColorHex>("fafad2");
        colors["lightgray"] = make_unique<ColorHex>("d3d3d3");
        colors["lightgreen"] = make_unique<ColorHex>("90ee90");
        colors["lightgrey"] = make_unique<ColorHex>("d3d3d3");
        colors["lightpink"] = make_unique<ColorHex>("ffb6c1");
        colors["lightsalmon"] = make_unique<ColorHex>("ffa07a");
        colors["lightseagreen"] = make_unique<ColorHex>("20b2aa");
        colors["lightskyblue"] = make_unique<ColorHex>("87cefa");
        colors["lightslategray"] = make_unique<ColorHex>("778899");
        colors["lightslategrey"] = make_unique<ColorHex>("778899");
        colors["lightsteelblue"] = make_unique<ColorHex>("b0c4de");
        colors["lightyellow"] = make_unique<ColorHex>("ffffe0");
        colors["lime"] = make_unique<ColorHex>("00ff00");
        colors["limegreen"] = make_unique<ColorHex>("32cd32");
        colors["linen"] = make_unique<ColorHex>("faf0e6");
        colors["magenta"] = make_unique<ColorHex>("ff00ff");
        colors["maroon"] = make_unique<ColorHex>("800000");
        colors["maroon2"] = make_unique<ColorHex>("7f0000");
        colors["maroon3"] = make_unique<ColorHex>("b03060");
        colors["mediumaquamarine"] = make_unique<ColorHex>("66cdaa");
        colors["mediumblue"] = make_unique<ColorHex>("0000cd");
        colors["mediumorchid"] = make_unique<ColorHex>("ba55d3");
        colors["mediumpurple"] = make_unique<ColorHex>("9370db");
        colors["mediumseagreen"] = make_unique<ColorHex>("3cb371");
        colors["mediumslateblue"] = make_unique<ColorHex>("7b68ee");
        colors["mediumspringgreen"] = make_unique<ColorHex>("00fa9a");
        colors["mediumturquoise"] = make_unique<ColorHex>("48d1cc");
        colors["mediumvioletred"] = make_unique<ColorHex>("c71585");
        colors["midnightblue"] = make_unique<ColorHex>("191970");
        colors["mintcream"] = make_unique<ColorHex>("f5fffa");
        colors["mistyrose"] = make_unique<ColorHex>("ffe4e1");
        colors["moccasin"] = make_unique<ColorHex>("ffe4b5");
        colors["navajowhite"] = make_unique<ColorHex>("ffdead");
        colors["navy"] = make_unique<ColorHex>("000080");
        colors["oldlace"] = make_unique<ColorHex>("fdf5e6");
        colors["olive"] = make_unique<ColorHex>("808000");
        colors["olivedrab"] = make_unique<ColorHex>("6b8e23");
        colors["orange"] = make_unique<ColorHex>("ffa500");
        colors["orangered"] = make_unique<ColorHex>("ff4500");
        colors["orchid"] = make_unique<ColorHex>("da70d6");
        colors["palegoldenrod"] = make_unique<ColorHex>("eee8aa");
        colors["palegreen"] = make_unique<ColorHex>("98fb98");
        colors["paleturquoise"] = make_unique<ColorHex>("afeeee");
        colors["palevioletred"] = make_unique<ColorHex>("db7093");
        colors["papayawhip"] = make_unique<ColorHex>("ffefd5");
        colors["peachpuff"] = make_unique<ColorHex>("ffdab9");
        colors["peru"] = make_unique<ColorHex>("cd853f");
        colors["pink"] = make_unique<ColorHex>("ffc0cb");
        colors["plum"] = make_unique<ColorHex>("dda0dd");
        colors["powderblue"] = make_unique<ColorHex>("b0e0e6");
        colors["purple"] = make_unique<ColorHex>("800080");
        colors["purple2"] = make_unique<ColorHex>("7f007f");
        colors["purple3"] = make_unique<ColorHex>("a020f0");
        colors["rebeccapurple"] = make_unique<ColorHex>("663399");
        colors["red"] = make_unique<ColorHex>("ff0000");
        colors["rosybrown"] = make_unique<ColorHex>("bc8f8f");
        colors["royalblue"] = make_unique<ColorHex>("4169e1");
        colors["saddlebrown"] = make_unique<ColorHex>("8b4513");
        colors["salmon"] = make_unique<ColorHex>("fa8072");
        colors["sandybrown"] = make_unique<ColorHex>("f4a460");
        colors["seagreen"] = make_unique<ColorHex>("2e8b57");
        colors["seashell"] = make_unique<ColorHex>("fff5ee");
        colors["sienna"] = make_unique<ColorHex>("a0522d");
        colors["silver"] = make_unique<ColorHex>("c0c0c0");
        colors["skyblue"] = make_unique<ColorHex>("87ceeb");
        colors["slateblue"] = make_unique<ColorHex>("6a5acd");
        colors["slategray"] = make_unique<ColorHex>("708090");
        colors["slategrey"] = make_unique<ColorHex>("708090");
        colors["snow"] = make_unique<ColorHex>("fffafa");
        colors["springgreen"] = make_unique<ColorHex>("00ff7f");
        colors["steelblue"] = make_unique<ColorHex>("4682b4");
        colors["tan"] = make_unique<ColorHex>("d2b48c");
        colors["teal"] = make_unique<ColorHex>("008080");
        colors["thistle"] = make_unique<ColorHex>("d8bfd8");
        colors["tomato"] = make_unique<ColorHex>("ff6347");
        colors["turquoise"] = make_unique<ColorHex>("40e0d0");
        colors["violet"] = make_unique<ColorHex>("ee82ee");
        colors["wheat"] = make_unique<ColorHex>("f5deb3");
        colors["white"] = make_unique<ColorHex>("ffffff");
        colors["whitesmoke"] = make_unique<ColorHex>("f5f5f5");
        colors["yellow"] = make_unique<ColorHex>("ffff00");
        colors["yellowgreen"] = make_unique<ColorHex>("9acd32");
    }
}
